Package: diffUTR
Type: Package
Title: diffUTR: Streamlining differential exon and 3' UTR usage
Version: 1.20.0
Depends: R (>= 4.0)
Authors@R: c(
	person("Pierre-Luc", "Germain", email="pierre-luc.germain@hest.ethz.ch", 
	  role=c("cre","aut"), comment = c(ORCID = "0000-0003-3418-4218")), 
	person("Stefan", "Gerber", email="gerberst@student.ethz.ch", role="aut") )
Description: 
    The diffUTR package provides a uniform interface and plotting functions for
    limma/edgeR/DEXSeq -powered differential bin/exon usage. It includes in 
    addition an improved version of the limma::diffSplice method. Most 
    importantly, diffUTR further extends the application of these frameworks to 
    differential UTR usage analysis using poly-A site databases.
Imports: S4Vectors, SummarizedExperiment, limma, edgeR, DEXSeq,
        GenomicRanges, Rsubread, ggplot2, rtracklayer, ComplexHeatmap,
        ggrepel, stringi, methods, stats, GenomeInfoDb, dplyr,
        matrixStats, IRanges, ensembldb, viridisLite
Suggests: BiocStyle, knitr, rmarkdown
biocViews: GeneExpression
BugReports: https://github.com/ETHZ-INS/diffUTR
VignetteBuilder: knitr
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/diffUTR
git_branch: RELEASE_3_23
git_last_commit: ba87eee
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:59:07 UTC; biocbuild
Author: Pierre-Luc Germain [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-3418-4218>),
  Stefan Gerber [aut]
Maintainer: Pierre-Luc Germain <pierre-luc.germain@hest.ethz.ch>
Built: R 4.6.0; ; 2026-04-29 04:40:35 UTC; unix
