% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{example_gene_annotation}
\alias{example_gene_annotation}
\title{Example gene annotation}
\value{
a `GRanges` object
}
\description{
An example gene annotation containing only a small subset of mouse genes.
}
